var posAZ = 5000;             // Imaginary azimuth value, corrected at zero point
var dirAZ = false;            // Azimuth motor direction, relé vypnuto
var posEL = 5000;             // Imaginární hodnota elevace, opraví se v nulovém bodě
var dirEL = false;            // Elevation motor direction, relé vypnuto
var orgAZ = true;             // Azimuth origin switch, imaginární hodnota
var orgEL = true;             // Elevation origin switch, imaginární

var Gpio = require('onoff').Gpio; //require onoff to control GPIO

var AZonoff = new Gpio(23, 'out'); //declare GPIO23 an output, AZ motor On/Off
var ELonoff = new Gpio(27, 'out'); //declare GPIO27 an output, EL motor On/Off
var AZdirection = new Gpio(4, 'out'); //declare GPIO4 an output, AZ smer
var ELdirection = new Gpio(17, 'out'); //declare GPIO17 an output, EL smer

var AZphaseA = new Gpio(20, 'in', 'rising'); //declare GPIO20 an input, AZ phase A
var AZphaseB = new Gpio(21, 'in', 'rising'); //declare GPIO21 an input, AZ phase B
var ELphaseB = new Gpio(26, 'in', 'rising'); //declare GPIO26 an input, EL phase B
var ELphaseA = new Gpio(19, 'in', 'rising'); //declare GPIO19 an input, EL phase A
var AZoriginS = new Gpio(12, 'in', 'both'); //declare GPIO12 an input, AZ origin switch
var ELoriginS = new Gpio(16, 'in', 'both'); //declare GPIO16 an input, EL origin switch

var fs = require('fs');    //require filesystem to read html files
var url = require('url');  // parsing dotazu

AZphaseA.watch(function (err, value) {  // Interrupt handling
   if (err) { //if an error
     console.error('There was an error AZphaseA', err); //output error message to console
     return;
     }
   if (dirAZ) {
     posAZ --; // respond to an interruption
     }
     else {
     posAZ ++; // respond to an interruption
     }
   });

AZphaseB.watch(function (err, value) {  // Interrupt handling
   if (err) { //if an error
    console.error('There was an error AZphaseB', err); //output error message to console
  return;
  }
  if (dirAZ) {
   posAZ --; // respond to an interruption
   }
   else {
   posAZ ++; // respond to an interruption
   }
});

ELphaseB.watch(function (err, value) {  // Interrupt handling
   if (err) { //if an error
    console.error('There was an error ELphaseB', err); //output error message to console
  return;
  }
   if (dirEL) {
   posEL --; // respond to an interruption
   }
   else {
   posEL ++; // respond to an interruption
   }
});

ELphaseA.watch(function (err, value) {  // Interrupt handling
   if (err) { //if an error
    console.error('There was an error ELphaseA', err); //output error message to console
  return;
  }
   if (dirEL) {
   posEL --; // respond to an interruption
   }
   else {
   posEL ++; // respond to an interruption
   }
});

AZoriginS.watch(function (err, value) {  // Interrupt handling
   if (err) { //if an error
    console.error('There was an error AZoriginS', err); //output error message to console
  return;
  }
   if (value) {
   orgAZ=true; // Switch open, true azimuth zero point
   posAZ=0;    // Set the actual default azimuth value    
   }
   else {
   orgAZ=false; // Switch closing
   }
});

ELoriginS.watch(function (err, value) {  // Interrupt handling
   if (err) { //if an error
    console.error('There was an error ELoriginS', err); //output error message to console
  return;
  }
   if (value) {
   orgEL=true; // Switch opening, true zero elevation point
   posEL=0;    // Set the actual default elevation value    
   }
   else {
   orgEL=false; // Switch closing
   }
});

function unexportOnClose() { //function to run when exiting program
  AZonoff.writeSync(0);      // Stop the AZ engine
  AZdirection.writeSync(0);  // Turns off the AZ direction relay
  ELonoff.writeSync(0);      // Stop the EL engine
  ELdirection.writeSync(0);  // Turns off the EL direction relay
  AZonoff.unexport();        // Unexport AZonoff to free resources
  AZdirection.unexport();    // Unexport AZdirection to free resources
  ELonoff.unexport();        // Unexport ELonoff to free resources
  ELdirection.unexport();    // Unexport ELdirection to free resources
  ELphaseA.unexport();       // Unexport ELphaseA to free resources
  ELphaseB.unexport();       // Unexport ELphaseB to free resources
  AZphaseB.unexport();       // Unexport AZphaseB to free resources
  AZphaseA.unexport();       // Unexport AZphaseA to free resources
  AZoriginS.unexport();      // Unexport AZoriginS to free resources
  ELoriginS.unexport();      // Unexport ELoriginS to free resources
  process.exit();            // end of program, on ctrl c
};

process.on('SIGINT', unexportOnClose); // Activates an interrupt from the value of the input pins

var http = require('http').createServer(function handler(req, res) { //create server

var q = url.parse(req.url, true); // go through and process the html query from the browser
  var  filename = "." + q.pathname;
  if (filename =="./") {filename = "./index.html"};
  var contentTypesByExtension = {
    '.html': "text/html",
    '.css':  "text/css",
    '.js':   "text/javascript"
    };
  fs.readFile(filename,  function(err, data) {
    if (err) { //if an error
       res.writeHead(404, {'Content-Type': 'text/html'}); //send error page
       return res.end("404 Not Found");
    }  
    res.writeHead(200);
    res.end(data);
  });
});

var io = require('socket.io')(http) //require socket.io module and pass the http object

http.listen(8080); //listen to port 8080; Here you can set the default port 80

io.sockets.on('connection', function (socket) { // WebSocket Connection
  var buttonState1 = 0; // variable to store button data1
  var buttonState2 = 0; // variable to store button data2
  
  socket.on('state', function (data1,data2) { //get button state from client
  buttonState1 = data1;
  buttonState2 = data2;
  switch (buttonState2) {
    case 10: 
        if (buttonState1 != AZonoff.readSync()) { // AZ was pressed -
        dirAZ = false;
          if (buttonState1 == 1) {
            AZonoff.writeSync(1); // Start the AZ engine
            }
            else
            {
            AZonoff.writeSync(0); // Stop the AZ engine
            console.log(posAZ + " ; " + posEL + " ; " + orgAZ + " ; " + orgEL);
            }
          }
        break;
    case 11:
        if (buttonState1 != AZonoff.readSync()) { // AZ was pressed +
        dirAZ = true;
          if (buttonState1 == 1) {
            AZdirection.writeSync(1); // Activate relay AZ direction
            AZonoff.writeSync(1); // Start the AZ engine
            }
            else
            {
            AZonoff.writeSync(0); // Stop the AZ engine
            AZdirection.writeSync(0); // Turns off the AZ direction relay
            console.log(posAZ + " ; " + posEL + " ; " + orgAZ + " ; " + orgEL);
            }
          }  
        break;          
    case 12: 
        if (buttonState1 != ELonoff.readSync()) { // EL was pressed -
        dirEL = false;
          if (buttonState1 == 1) {
            ELonoff.writeSync(1); // Start the EL engine
            }
            else
            {
            ELonoff.writeSync(0); // Stop the EL engine
            console.log(posAZ + " ; " + posEL + " ; " + orgAZ + " ; " + orgEL);
            }
          }
        break;
    case 13: 
        if (buttonState1 != ELonoff.readSync()) { // EL was pressed +
        dirEL = true;
          if (buttonState1 == 1) {
            ELdirection.writeSync(1); // Activate relay EL direction
            ELonoff.writeSync(1); // Start the EL engine
            }
            else
            {
            ELonoff.writeSync(0); // Stop the EL engine
            ELdirection.writeSync(0); // Turns off the EL direction relay
            console.log(posAZ + " ; " + posEL + " ; " + orgAZ + " ; " + orgEL);
            }
          }  
        break;
    }
  });
});
